/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm;

import java.util.List;
import java.util.Map;

import org.jbpm.deploy.Deployment;
import org.jbpm.env.session.Job;
import org.jbpm.pvm.Execution;
import org.jbpm.pvm.ProcessDefinition;


/** session facade interface for the Process Virtual Machine.
 * @author Tom Baeyens
 */
public interface PvmService {

  /** deploy a new process definition into the process repository. */
  ProcessDefinition deploy(Deployment deployment);

  /** all deployed process names. */
  List<String> findProcessDefinitionNames();

  /** all versions of the given process. */
  List<ProcessDefinition> findProcessDefinitions(String processDefinitionName);

  /** latest version of the processDefinition with the given name. */
  ProcessDefinition findLatestProcessDefinition(String processDefinitionName);

  /** specific version of a named processDefinition. */
  ProcessDefinition findProcessDefinition(String processDefinitionName, int processDefinitionVersion);

  /** specific processDefinition by database primary key. */
  ProcessDefinition findProcessDefinition(long processDefinitionDbid);

  /** starts a new execution in the latest version of the given process definition. */
  Execution startExecution(String processDefinitionName);

  /** starts a new execution in the exact version indicated by processDefinitionDbid. */
  Execution startExecution(long processDefinitionDbid);
  
  /** starts a new execution in the latest version of the given process definition. */ 
  Execution startExecution(String processDefinitionName, Map<String, Object> executionVariables);
  
  /** starts a new execution in the exact version indicated by processDefinitionDbid. */
  Execution startExecution(long processDefinitionDbid, Map<String, Object> executionVariables);

  /** starts a new execution in the latest version of the given process definition. */ 
  Execution startExecution(String processDefinitionName, Map<String, Object> executionVariables, String executionKey);

  /** starts a new execution in the exact version indicated by processDefinitionDbid. */
  Execution startExecution(long processDefinitionDbid, Map<String, Object> executionVariables, String executionKey);
  
  /** the execution with the given database primary key */
  Execution findExecution(long executionDbid);

  /** the execution that is uniquely defined by the process definition and the business key. */
  Execution findExecution(String processDefinitionName, String key);

  /** provides an external trigger to an execution. */
  Execution signal(long executionDbid);
  
  /** provides a named external trigger to an execution. */
  Execution signal(long executionDbid, String signalName);
  
  /** provides a named external trigger to an execution with parameters. */
  Execution signal(long executionDbid, String signalName, Map<String, Object> parameters);
  
  /** creates or overwrites a variable value on the referenced execution */ 
  Execution setVariable(long executionDbid, String name, Object value);
  
  /** creates or overwrites the variable values on the referenced execution */ 
  Execution setVariables(long executionDbid, Map<String, Object> variables);
  
  /** retrieves a variable */ 
  Object getVariable(long executionDbid, String variableName);
  
  /** retrieves a map of variables */ 
  Map<String, Object> getVariables(long executionDbid, List<String> variableNames);
  
  /** fetches all jobs ordered in ascending dueDate. */
  List<Job<?>> getJobs();
}
