/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jbpm.env.Environment;
import org.jbpm.env.session.PvmDbSession;
import org.jbpm.pvm.Execution;
import org.jbpm.pvm.impl.ExecutionImpl;

/**
 * @author Tom Baeyens
 */
public class GetVariables implements Command<Map<String, Object>> {

  private static final long serialVersionUID = 1L;
  
  protected long executionDbid;
  protected List<String> variableNames;
  
  public GetVariables(long executionDbid, List<String> variableNames) {
    this.executionDbid = executionDbid;
    this.variableNames = variableNames;
  }

  public Map<String, Object> execute(Environment environment) throws Exception {
    Execution execution = environment.get(PvmDbSession.class).get(ExecutionImpl.class, executionDbid);
    
    Map<String, Object> variables = new HashMap<String, Object>();
    for (String variableName : variableNames) {
      Object value = execution.getVariable(variableName);
      variables.put(variableName, value);
    }

    return variables;
  }
}
