/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.client;

import java.util.Map;

import org.jbpm.env.Environment;
import org.jbpm.pvm.Execution;


/**
 * @author Tom Baeyens
 */
public class SetVariables extends VariablesCommand<Execution> {
  
  private static final long serialVersionUID = 1L;
  
  protected long executionId;
  
  public SetVariables() {
  }

  public SetVariables(long executionId) {
    this.executionId = executionId;
  }

  public SetVariables(long executionId, Map<String, Object> variables) {
    this.executionId = executionId;
    this.variables = variables;
  }

  public SetVariables(long executionId, String key, Object variable) {
    this.executionId = executionId;
    addVariable(key, variable);
  }

  public Execution execute(Environment environment) throws Exception {
    Execution execution = getExecution(environment, executionId);
    execution.setVariables(variables);
    return execution;
  }

  public long getExecutionId() {
    return executionId;
  }
  public void setExecutionId(long executionId) {
    this.executionId = executionId;
  }
}
