/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.env.session;

import java.util.List;

import org.jbpm.pvm.Execution;
import org.jbpm.pvm.ProcessDefinition;

/**
 * @author Tom Baeyens, Pascal Verdage
 */
public interface PvmDbSession extends DbSession {

  /** all deployed process names. */
  List<String> findProcessDefinitionNames();

  /** all versions of the given process. */
  List<ProcessDefinition> findProcessDefinitions(String name);

  /** latest version of the processDefinition with the given name. */
  ProcessDefinition findProcessDefinition(String name);

  /** specific version of a named processDefinition. */
  ProcessDefinition findProcessDefinition(String name, int version);

  /** next timer to execute */
  Timer findNextTimer();

  /** all scheduled timers */
  List<Timer> findAllTimers();

  /** all timers scheduled on a given execution */
  List<Timer> findTimers(Execution execution);

  /** an execution identified by process definition and business key */ 
  Execution findExecution(String processDefinitionName, String key);

  /** all jobs */
  List<Job<?>> getJobs();
}
