/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.jobexecutor;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.jbpm.client.Command;
import org.jbpm.client.CommandService;
import org.jbpm.env.Environment;
import org.jbpm.env.session.Job;
import org.jbpm.util.Listener;

/**
 * @author Tom Baeyens
 */
public class JobExceptionHandler implements Listener, Command<Object> {
  private static final long serialVersionUID = 1L;
  CommandService commandService;
  long jobDbid;
  Throwable exception;
  
  public JobExceptionHandler(long jobDbid, Throwable exception, CommandService commandService) {
    this.commandService = commandService;
    this.jobDbid = jobDbid;
    this.exception = exception;
  }

  public void event(Object source, String eventName, Object info) {
    commandService.execute(this);
  }

  public Object execute(Environment environment) throws Exception {
    // load the job from the db
    Job<?> job = environment.get(JobSession.class).get(jobDbid);
    // serialize the stack trace
    StringWriter sw = new StringWriter();
    exception.printStackTrace(new PrintWriter(sw));
    if (job != null) {
	    job.setException(sw.toString());
	    // decrement the number of retries
	    job.setRetries(job.getRetries()-1);
    }
    return null;
  }
  
}
