/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.jobexecutor;

import org.jbpm.log.Log;

import org.jbpm.env.Environment;
import org.jbpm.env.session.Message;
import org.jbpm.env.session.MessageSession;
import org.jbpm.tx.StandardTransaction;
import org.jbpm.tx.Transaction;
import org.jbpm.util.Listener;
import org.jbpm.util.ReflectUtil;

/**
 * @author Tom Baeyens
 */
public class StandardMessageSession implements MessageSession {
  
  private static final Log log = Log.getLog(StandardMessageSession.class.getName());

  public void send(Message<?> message) {
    log.trace("sending message "+ReflectUtil.getUnqualifiedClassName(message.getClass()));

    Environment.getCurrent().get(JobSession.class).save(message);

    final JobExecutor jobExecutor = Environment.getCurrent().get(JobExecutor.class);
    // notify the job executor after the transaction is completed
    Listener jobExecutorNotifier = new Listener() {
      public void event(Object source, String eventName, Object info) {
        if ( (Transaction.EVENT_AFTERCOMPLETION.equals(eventName))
             && (StandardTransaction.STATE_COMMITTED.equals(info))
           ) {
          jobExecutor.jobWasAdded();
        }
      }
    };
    Transaction transaction = Environment.getCurrent().getTransaction();
    transaction.addListener(jobExecutorNotifier);
    
    log.debug("registered notification listener with "+transaction);
  }
  
}
