/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm;

import java.io.Serializable;
import java.util.List;
import java.util.Set;

/** common base interface for all process definition objects that can have 
 * exception handlers and properties.
 * 
 * @author Tom Baeyens 
 */
public interface ProcessElement extends Serializable {

  /** the meaningless database primary key */
  long getDbid();

  /** the process definition to which this process element belongs */
  ProcessDefinition getProcessDefinition();

  /** the property value for the given key or null if no such 
   * configuration key is present.
   * 
   * This is extra static configuration information that can be 
   * associated to a process element.  Process languages can use this 
   * to store configuration properties for extensions in it.  That way, 
   * these extensions can be added without changing the process model 
   * or the database schema. Examples of extensions that may use these 
   * configuration properties are simulation, predictive scheduling, 
   * graphical information,... */
  Object getProperty(String key);
  
  /** the set of available property keys or an empty set in case 
   * there are no keys. */
  Set<String> getPropertyKeys();

  /** the list of exception handlers that can be tried if an exception 
   * occurs in user code that is associated to this process element or a 
   * process element inside of this process element. 
   * Beware: the actual member is returned.  No copy is made. */
  List<? extends ExceptionHandler> getExceptionHandlers();
}
