/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm;

import org.jbpm.pvm.type.Converter;

/** declaration of a variable.
 * 
 * @author Tom Baeyens
 */
public interface VariableDefinition {

  /** the meaningless database primary key */
  long getDbid();

  /** the name/key of the variable */ 
  String getName();
  
  /** free text description of this variable */
  String getDescription();
  
  /** defines the {@link Converter conversion} between the java object 
   * representation and and the storage value as part of the type 
   * declaration for this variable */
  Converter getConverter();
  
  /** defines the persistable variable type that is part of the type 
   * declaration for this variable */
  Class<?> getVariableClass();

  /** generates the initial value for this variable declaration in the 
   * context of the given execution. */ 
  Object getInitialValue(Execution execution);
}
