/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.impl;

import java.util.logging.Level;
import org.jbpm.log.Log;

import org.jbpm.env.session.Message;
import org.jbpm.pvm.Activity;
import org.jbpm.pvm.impl.ExecutionImpl.Propagation;

public class ExecuteNodeOp implements AtomicOperation {
  
  private static Log log = Log.getLog(ExecuteNodeOp.class.getName());
  
  public boolean isAsync(ExecutionImpl execution) {
    return execution.getNode().isExecutionAsync();
  }

  public void perform(ExecutionImpl execution) {
    NodeImpl node = execution.getNode();
    
    if (log.isTraceEnabled()) {
      if (execution.getName()!=null) {
        log.trace(execution.toString()+" executes "+node);
      } else {
        log.trace("executing "+node);
      }
    }
    
    Activity activity = node.getBehaviour();
    
    try {
      execution.userCodeType = ExecutionImpl.UserCodeType.NODE_BEHAVIOUR;
      execution.propagation = Propagation.UNSPECIFIED;
      activity.execute(execution);
    } catch (Exception e) {
      execution.handleException(node, null, null, e, "couldn't execute node behaviour "+activity);
    } finally {
      execution.userCodeType = null;
    }
    
    if (execution.propagation == Propagation.UNSPECIFIED) {
      execution.proceed();
    }
  }

  public String toString() {
    return "execute(node)";
  }

  public Message<?> createAsyncMessage(ExecutionImpl execution) {
    return new ExecuteNodeMessage(execution);
  }
}