/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.type.matcher;

import org.jbpm.log.Log;

import org.hibernate.SessionFactory;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.type.LongType;
import org.jbpm.env.Environment;
import org.jbpm.pvm.type.Matcher;


public class HibernateLongIdMatcher implements Matcher {

  private static final long serialVersionUID = 1L;

  public boolean matches(String name, Object value) {
    boolean matches = false;
    
    Environment environment = Environment.getCurrent();
    if (environment!=null) {
      SessionFactory sessionFactory = (SessionFactory) environment.get("hibernate.session.factory");
      if (sessionFactory!=null) {
        ClassMetadata classMetadata = sessionFactory.getClassMetadata(value.getClass());
        matches =  ( (classMetadata!=null)
                     && (classMetadata.getIdentifierType().getClass()==LongType.class)
                   );
      }
    } else {
      log.trace("no current environment so valueClass cannot be stored as a long-id-ref to a hibernate object");
      matches = false;
    }
    return matches;
  }
  
  private static Log log = Log.getLog(HibernateLongIdMatcher.class.getName());
}
