/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.tx;

import org.jbpm.util.Listener;
import org.jbpm.util.Observable;

/**
 * @author Tom Baeyens
 */
public interface Transaction extends Listener, Observable {

  /**
   * is fired when the wiring environment is being flushed.  No event info provided. 
   */
  String EVENT_FLUSH = "flush";

  /**
   * is fired when the wiring environment is being committed.  No event info provided.  
   * It is the Transaction that will listen to the close of this wiring environment.
   * Upon close, the transaction will fire a commit or rollback event depending on 
   * the state of the isRollback property in the transaction.
   */
  String EVENT_BEFORECOMPLETION = "before-completion";

  /**
   * is fired when the wiring environment is being rolled back.  No event info provided.  
   * It is the Transaction that will listen to the close of this wiring environment.
   * Upon close, the transaction will fire a commit or rollback event depending on 
   * the state of the isRollback property in the transaction.
   */
  String EVENT_AFTERCOMPLETION = "after-completion";

  /** marks a transaction for rollback only. 
   * The transaction will be rolled back upon closing the environment. */
  void setRollbackOnly();

  boolean isRolledBack();
  boolean isCommitted();
  boolean isRollbackOnly();
}
