/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.wire.binding;

import org.jbpm.wire.descriptor.AbstractDescriptor;
import org.jbpm.wire.descriptor.CharacterDescriptor;
import org.jbpm.wire.xml.WireParser;
import org.jbpm.xml.Binding;

/**
 *
 * <p>This {@link Binding} specifies a {@link CharacterDescriptor}.</p>
 *
 * <p>A character descriptor is defined by a <b><code>{@literal <char>}</code></b> xml element.</p>
 *
 * <p>This element can have an attribute "name", which specifies the name of the created object in the WireContext.</p>
 * <p>This element must have an attribute "value", which specifies the value of the object. This value should be a single char.</p>
 *
 * <h3>Example</h3>
 *
 * The following Xml declaration will create a Character 'c', with the value 'a'.
 * <pre> &lt;objects&gt;
 *   &lt;char name='c' value='a' /&gt;
 * &lt;/objects&gt;</pre>
 *
 *
 * @author Tom Baeyens
 * @author Guillaume Porcher (documentation)
 *
 * @see WireParser
 */
public class CharBinding extends BasicTypeBinding {

  protected AbstractDescriptor createDescriptor(String value) {
    CharacterDescriptor characterDescriptor = new CharacterDescriptor();
    if (value.length()!=1) {
      throw new IllegalArgumentException("length of value must be 1");
    }
    char c = value.charAt(0);
    characterDescriptor.setValue(new Character(c));
    return characterDescriptor;
  }
}