/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.wire.descriptor;

import org.jbpm.log.Log;

import org.jbpm.env.Environment;
import org.jbpm.wire.WireContext;
import org.jbpm.wire.WireException;

/**
 * @author Tom Baeyens
 */
public class TypeRefDescriptor extends AbstractDescriptor {

  private static final long serialVersionUID = 1L;
  private static final Log log = Log.getLog(TypeRefDescriptor.class.getName());
  
  Class<?> environmentClass;
  Class<?> contextClass;
  
  public Object construct(WireContext wireContext) {
    if (contextClass!=null) {
      log.trace("looking up "+contextClass.getName()+" by type in "+wireContext);
      return wireContext.get(contextClass);

    } else if (environmentClass!=null) {
      Environment environment = wireContext.getEnvironment();
      if (environment==null) {
        throw new WireException("no environment to search an object of type "+environmentClass.getName());
      }
      log.trace("looking up "+environmentClass.getName()+" by type in environment");
      return environment.get(environmentClass);
    }
    return null;
  }

  public void setEnvironmentClass(Class< ? > environmentClass) {
    this.environmentClass = environmentClass;
  }
  public void setContextClass(Class< ? > contextClass) {
    this.contextClass = contextClass;
  }
}
