/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

import org.ow2.novabpm.util.Misc.NullCheckResult;

/**
 * @author Pierre Vigneras
 * @date Feb 7, 2008
 */
public class MiscTest extends TestCase {

  
  /**
   * @author Pierre Vigneras
   * @date Mar 19, 2008
   */
  public interface ClonedObject {

  }
  public static class ChainMock { 
    private List<Object> passedBy = new ArrayList<Object>();
    public List<Object> getPassedBy() {
      return passedBy;
    }
    void passedBy(final Object o) {
      passedBy.add(o);
    }
  }
  public static interface ChainTestOp {
    void trace(ChainMock mock);
  }
  public abstract static class AbstractChainTest implements ChainTestOp { 
    public void trace(ChainMock mock) { 
      mock.passedBy(this);
    }
  }
  public static class ConcreteChainTestA extends AbstractChainTest { }
  public static class ConcreteChainTestB implements ChainTestOp { 
    public void trace(ChainMock mock) { 
      mock.passedBy(this);
    }
  }
  
  public void testGetChainOf() {
    final List<ChainTestOp> l = new LinkedList<ChainTestOp>();
    final ChainTestOp a = new ConcreteChainTestA();
    final ChainTestOp b = new ConcreteChainTestB();
    l.add(a);
    l.add(b);
    final ChainTestOp p = Misc.getChainOf(l);
    final ChainMock mock = new ChainMock();
    p.trace(mock);
    assertTrue(mock.getPassedBy().containsAll(l));
  }
  
  
//  public void testDeepToString() {
//    System.out.println(Misc.deepToString(null));
//    System.out.println(Misc.deepToString(true));
//    byte b = 127;
//    System.out.println(Misc.deepToString(b));
//    System.out.println(Misc.deepToString('a'));
//    short sh = 32000;
//    System.out.println(Misc.deepToString(sh));
//    int i = 10240000;
//    System.out.println(Misc.deepToString(i));
//    long l = i*i*i;
//    System.out.println(Misc.deepToString(l));
//    float f = 3.14f;
//    System.out.println(Misc.deepToString(f));
//    double d = Math.E;
//    System.out.println(Misc.deepToString(d));
//    System.out.println(Misc.deepToString(Void.TYPE));
//    System.out.println(Misc.deepToString("Hello World"));
//    System.out.println(Misc.deepToString(new Object()));
//    System.out.println(Misc.deepToString(new Object() {
//      private String s = "Dummy Field";
//    }));
//    System.out.println(Misc.deepToString(new Object() {
//      class DummyClass {
//        String name = "DummyName";
//        class SecondDummyClass extends HashMap<String, String> {
//          
//        }
//        SecondDummyClass secondField = new SecondDummyClass();
//      }
//      DummyClass dummyClass = new DummyClass();
//    }));
//    System.out.println(Misc.deepToString(new Final<String>()));
//    System.out.println(Misc.deepToString(DummyEnum1.ONE));
//    System.out.println(Misc.deepToString(new Object() {
//      DummyEnum1 e = DummyEnum1.ONE;
//      Inner i = new Inner();
//      class Inner {
//        boolean[] tbo = new boolean[5];
//        byte[] tby = new byte[5];
//        char[] tch = new char[5]; 
//        short[] tsh = new short[5];
//        int[] tin = new int[5];
//        long[] tlo = new long[5];
//        float[] tfl = new float[5];
//        double[] tdo = new double[5];
//        Object[] to = new Object[5];
//        Inner() {
//          Arrays.fill(tbo, true);
//          Arrays.fill(tby, (byte) 10);
//          Arrays.fill(tch, 'a');
//          Arrays.fill(tsh, (short) 32000);
//          Arrays.fill(tin, 10000000);
//          Arrays.fill(tlo, 10000000*1000000);
//          Arrays.fill(tfl, (int) 1.0);
//          Arrays.fill(tdo, Math.E);
//          Arrays.fill(to, "Hey");
//        }
//      }
//    }));
//  }
//  
//  static enum DummyEnum1 { ONE, TWO, THREE };
  

	/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#findAllInterfaces(java.lang.Class)}.
	 */
	public void testFindAllInterfaces() {
		assertTrue(Misc.findAllInterfaces(ChainTestOp.class).size() == 0);
		assertTrue(Misc.findAllInterfaces(AbstractChainTest.class).size() == 1);
		assertTrue(Misc.findAllInterfaces(ConcreteChainTestA.class).size() == 1);
		assertTrue(Misc.findAllInterfaces(ConcreteChainTestB.class).size() == 1);
	}


/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#getAllContentFrom(java.io.Reader)}.
	 * @throws IOException 
	 */
	public void testGetAllContentFromReader() throws IOException {
	  final byte[] src = "Hello World".getBytes();
	  final ByteArrayInputStream bais = new ByteArrayInputStream(src);
	  final byte[] result = Misc.getAllContentFrom(bais);
	  assertTrue(Arrays.equals(src, result));
	}

	/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#findNull(java.lang.Object[])}.
	 */
	public void testFindNull() {
		NullCheckResult result = Misc.findNull(null);
		assertTrue(result.hasNull());
		assertTrue(result.isNull(0));

		result = Misc.findNull("1");
		assertFalse(result.hasNull());

		result = Misc.findNull(null, null);
		assertTrue(result.hasNull());
		assertTrue(result.isNull(0));
		assertTrue(result.isNull(1));

		result = Misc.findNull("1", null);
		assertTrue(result.hasNull());
		assertFalse(result.isNull(0));
		assertTrue(result.isNull(1));

		result = Misc.findNull(null, "2");
		assertTrue(result.hasNull());
		assertTrue(result.isNull(0));
		assertFalse(result.isNull(1));
		

		result = Misc.findNull("1", "2");
		assertFalse(result.hasNull());

	}

	/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#checkArgsNotNull(java.lang.Object[])}.
	 */
	public void testCheckArgsNotNull() {
		try {
		  dummyMethod(null, "notNull", null);
		  fail("Should throw an IllegalArgumentException here!");
		}catch(IllegalArgumentException iae) {
		  // ok
		}catch(Exception e) {
		  e.printStackTrace();
		  fail("Should never throw any other exception than IllegalArgumentException");
		}
	}

	private void dummyMethod(Object o, String s, Misc m) {
	  Misc.checkArgsNotNull(o, s, m);
	}
	/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#getStringFrom(java.util.BitSet, java.lang.String[])}.
	 */
	public void testGetStringFrom() {
		NullCheckResult result = Misc.findNull(null);
		List<String> list = Misc.getStringFrom(result, "1");
		assertTrue(list.size() == 1);
		assertTrue(list.contains("1"));
		
		result = Misc.findNull("1");
		list = Misc.getStringFrom(result, "1");
		assertTrue(list.size() == 0);
		
		result = Misc.findNull(null, null);
		list = Misc.getStringFrom(result, "1", "2");
		assertTrue(list.size() == 2);
		assertTrue(list.contains("1"));
		assertTrue(list.contains("2"));
		
		result = Misc.findNull("1", null);
		list = Misc.getStringFrom(result, "1", "2");
		assertTrue(list.size() == 1);
		assertFalse(list.contains("1"));
		assertTrue(list.contains("2"));
		
		
		result = Misc.findNull(null, "2");
		list = Misc.getStringFrom(result, "1", "2");
		assertTrue(list.size() == 1);
		assertTrue(list.contains("1"));
		assertFalse(list.contains("2"));
	}

	/**
	 * Test method for {@link org.ow2.novabpm.util.Misc#badStateIfNull(java.lang.Object, java.lang.String)}.
	 */
	public void testBadStateIf() {
		try {
		  Misc.badStateIfTrue(true, "dummy");
		  fail("Should throw an IllegalStateException");
		}catch(IllegalStateException ise) {
		  // ok
		}
		Misc.badStateIfTrue(false, "Dummy");
		// no problem
		Misc.badStateIfFalse(true, "dummy");
		// no problem
		try {
		  Misc.badStateIfFalse(false, "dummy");
      fail("Should throw an IllegalStateException");
    }catch(IllegalStateException ise) {
      // ok
    }
    try {
    Misc.badStateIfNull(null, "dummy");
    fail("Should throw an IllegalStateException");
    }catch(IllegalStateException ise) {
      // ok
    }
    Misc.badStateIfNull("Ok", "dummy");
    // no problem
    Misc.badStateIfEquals("a", "b", "dummy");
    // no problem
    try {
      Misc.badStateIfEquals("a", "a", "dummy");
      fail("Should throw an IllegalStateException");
    }catch(IllegalStateException ise) {
      // ok
    }
	}
	
	public void testDeleteDir() throws IOException {
	  File dir = null;
	  try {
	    Misc.deleteDir(dir);
	    fail("Check for null argument!");
	  } catch (IllegalArgumentException e) {
	    // ok
	  }
	  try {
	    dir = File.createTempFile("MiscTest", "tmp");
	    Misc.deleteDir(dir);
	    fail("Check that the given file is a directory!");
	  } catch (IllegalArgumentException e) {
	    assertTrue(e.getMessage().contains(dir.getName()));
	  }
	  dir = new File(Misc.TMP_DIR, Misc.getUniqueId("MiscTest-") + Misc.getRandomString(5));
	  Misc.deleteDir(dir);
	  // Should not fail!
	  dir = new File(Misc.TMP_DIR, Misc.getUniqueId("MiscTest-") + Misc.getRandomString(5));
	  assertTrue(dir.mkdirs());
	  populate(dir, Misc.random(5, 10));
	  assertTrue(dir.exists() && dir.isDirectory());
	  assertTrue(dir.listFiles().length > 0);
	  Misc.deleteDir(dir);
	  assertFalse(dir.exists());
	}


  /**
   * @param dir
   * @throws IOException 
   */
  private void populate(final File dir, int n) throws IOException {
    while(n-- > 0) {
      final boolean newDir = Misc.RANDOM.nextBoolean();
      final String name = newDir ? n + "" : n + ".tmp";
      final File file = new File(dir, name);
      if (newDir) {
        file.mkdir();
        populate(file, n - 1);
      } else {
        file.createNewFile();
      }
    }
  }
  
  public void testRandom() {
    final int n = Misc.RANDOM.nextInt(1000);
    for (int i = 0; i < n; i++) {
      int p;
      int k;
      do {
        p = Misc.RANDOM.nextInt(1000);
        k = Misc.RANDOM.nextInt(1000);
      } while (p == k);
//      System.out.println(p + ", " + k);
      final int min = Math.min(p, k);
      final int max = Math.max(p, k);
      final int r = Misc.random(min, max);
      assertTrue(r + ">" + min, r >= min);
      assertTrue(r + "<" + min, r <= max);
    }
  }
  
  public void testUnreachableStatement() {
    try {
      Misc.unreachableStatement();
    } catch(IllegalStateException ise) {
      // ok
    }
  }
  
//  public void testAddAllDeep() {
//    final Collection<ClonedObject> src = new ArrayList<ClonedObject>();
//    Misc.addAllDeep(src, new HashSet<ClonedObject>(), new Cloner<ClonedObject>() {
//
//      public ClonedObject newClone(ClonedObject t) {
//        // TODO Auto-generated method stub
//        return null;
//      }
//      
//    });
//  }
}
